package com.example.backend;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.backend.categories.model.CategorieEntity;
import com.example.backend.categories.service.CategorieService;
import com.example.backend.core.errors.NotFoundException;
import com.example.backend.viewed.model.ViewedEntity;
import com.example.backend.movies.model.MovieEntity;
import com.example.backend.movies.service.MovieService;
import com.example.backend.users.model.UserEntity;
import com.example.backend.users.service.UserService;
import com.example.backend.viewed.service.ViewedService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class MovieServiceTest {
    @Autowired
    private MovieService movieService;

    private MovieEntity movie;
    private MovieEntity movie2;

    @Autowired
    private CategorieService categorieService;

    private CategorieEntity cat;

    @Autowired
    private UserService userService;

    private UserEntity user;
    private UserEntity user1;
    private UserEntity user2;
    private UserEntity user3;

    @Autowired
    private ViewedService viewedService;

    @BeforeEach
    void createData() {
        removeData();

        cat = categorieService.create(new CategorieEntity(0, "Drama", "null"));

        user = userService.create(new UserEntity(0, "1", "1", true));
        user1 = userService.create(new UserEntity(0, "2", "2", true));
        user2 = userService.create(new UserEntity(0, "3", "3", true));
        user3 = userService.create(new UserEntity(0, "4", "4", true));

        movie = movieService.create(new MovieEntity(0, cat, "1", "1", "1", "1"));
        movie2 = movieService.create(new MovieEntity(0, cat, "2", "2", "2", "2"));

        viewedService.create(new ViewedEntity(0, user, movie));
        viewedService.create(new ViewedEntity(0, user1, movie));
        viewedService.create(new ViewedEntity(0, user2, movie));
        viewedService.create(new ViewedEntity(0, user3, movie));

    }

    @AfterEach
    void removeData() {
        viewedService.getAll(0).forEach(vs -> viewedService.delete(vs.getId()));
        userService.getAll().forEach(u -> userService.delete(u.getId()));
        movieService.getAll(0).forEach(item -> movieService.delete(item.getId()));
        categorieService.getAll().forEach(item -> categorieService.delete(item.getId()));
    }

    @Test
    @Order(1)
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> movieService.get(10));
    }

    @Test
    @Order(2)
    void createTest() {
        Assertions.assertEquals(2, movieService.getAll(cat.getId()).size());
        Assertions.assertEquals(movie, movieService.get(movie.getId()));
    }

    @Test
    @Order(3)
    void createNotUniqueTest() {
        final MovieEntity nonUniqueCategorie = new MovieEntity(3, cat, "1", "1", "1",
                "1");
        Assertions.assertThrows(IllegalArgumentException.class, () -> movieService.create(nonUniqueCategorie));
    }

    @Test
    @Order(4)
    void createNullableTest() {
        final MovieEntity nullableCategorie = new MovieEntity(1, cat, null, "1", "1",
                "1");
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> movieService.create(
                nullableCategorie));
    }

    @Test
    @Order(5)
    void updateTest() {
        final String newName = "TEST";
        final String oldName = movie.getName();
        final MovieEntity mov = new MovieEntity(movie.getId(), movie.getCategorie(),
                newName, movie.getDescription(),
                movie.getDuration(), movie.getImage());
        final MovieEntity newEntity = movieService.update(movie.getId(), mov);
        Assertions.assertEquals(newName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    @Order(6)
    void deleteTest() {
        movieService.delete(movie2.getId());
        Assertions.assertEquals(1, movieService.getAll(cat.getId()).size());

        Assertions.assertNotEquals(3, movieService.getAll(cat.getId()).size());
    }

    @Test
    @Order(7)
    void countViewedTest() {
        Assertions.assertEquals(movieService.countView(movie.getId()), 4);
    }
}
